package es.fileStore.business.db.impl;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.io.Serializable;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.annotation.Resource;

import org.apache.commons.io.IOUtils;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import es.fileStore.bean.FiltroFicheros;

import es.fileStore.business.db.IFicherosLN;
import es.fileStore.entity.Ficheros;
import es.fileStore.utils.ConfigurationManager;
import es.fileStore.view.bean.FicheroBean;
import es.fileStore.view.bean.UploadedFile;
import es.fileStore.dao.IFicherosDAO;

public class FicherosLNImpl extends GenericBusinessLogicImpl<Ficheros, Serializable> implements IFicherosLN {

	private final String NOMBRE_FICHERO 					= "nombreFichero";
	private final String EXTENSION_FICHERO 					= "extensionFichero";
	
	@Resource(name="ficherosDAO")
	private IFicherosDAO ficherosDAO;
	
	@Override
	@Transactional(propagation=Propagation.REQUIRED, rollbackFor=Exception.class)
	public List<FicheroBean> consulta(FiltroFicheros filtro) {
		return this.ficherosDAO.consulta(filtro);
	}

	@Override
	@Transactional(propagation=Propagation.REQUIRED, rollbackFor=Exception.class)
	public void altaFichero(Ficheros fichero,UploadedFile ficheroSubido) throws Exception {
		
		DateFormat df = new SimpleDateFormat("ddMMyyyyHHmmssSS");
		String date = df.format(new Date());
		Map<String,String> nombreYextension = this.extraerExtensionNombreFichero(ficheroSubido.getFileName());
		String nombre = nombreYextension.get(NOMBRE_FICHERO);
		String extension = nombreYextension.get(EXTENSION_FICHERO);
		//si el fichero tiene extensin: nombre_fecha.extension
		//si el fichero no tiene extension: nombre_fecha 
		String nombreFichero = extension==null?(nombre+"_"+date):(nombre+"_"+date+"."+extension);
		
		fichero.setRutaFichero(nombreFichero);
		this.ficherosDAO.altaFichero(fichero);
		//ahora, una vez registrado el envo del fichero en bd, lo guardamos fisicamente en el servidor
		String raizFicheros = ConfigurationManager.getProperty("ruta_ficheros");
		
		
		File directorioUbicacion = null;
		
		File outFile = null;
		//Stream de lectura del fichero subido
		
		FileOutputStream outAguardar = null;

		InputStream inSubido = ficheroSubido.getInputStream();
		
		ByteArrayOutputStream out = new ByteArrayOutputStream(inSubido.available());
		
		IOUtils.copy(inSubido, out);
		
		if ( inSubido != null ) {
			inSubido.close();
		}
		
								
		//creamos el directorio de ubicacin del fichero si no existe
		
		directorioUbicacion = new File(raizFicheros);
		if(!directorioUbicacion.exists()) {
					
			directorioUbicacion.mkdirs();
		}
		
		outFile = new File(directorioUbicacion,nombreFichero);
		outAguardar = new FileOutputStream(outFile);

		IOUtils.write(out.toByteArray(), outAguardar);
				
		outAguardar.flush();
		outAguardar.close();
				
		if ( outAguardar != null ) {
			outAguardar.close();
		}
		
		
	}
	
	@Override
	@Transactional(propagation=Propagation.REQUIRED, rollbackFor=Exception.class)
	public Ficheros detalle(Integer idFichero) {
		return this.ficherosDAO.detalle(idFichero);
	}
	
	
	/**
	 * Devuelve un mapa con el nombre del fichero y la extensin. Por ejemplo, para el nombre
	 * 'fichero1.txt' el nombre sera 'fichero1' y la extensin 'txt'. Para recuperar estos valores
	 * usaremos las constantes NOMBRE_FICHERO y EXTENSION_FICHERO
	 * @param nombreFichero
	 * @return
	 */
	private Map<String,String> extraerExtensionNombreFichero(String nombreFichero){
		Map<String,String> map = new HashMap<String,String>();
		//buscamos el punto que indica la extensin del fichero
		int lastDot = nombreFichero.lastIndexOf(".");
		String nombre = nombreFichero;
		String extension = null;
		if(-1 != lastDot){
			extension = nombreFichero.substring(lastDot+1);
			nombre = nombreFichero.substring(0,lastDot);
		}
		map.put(NOMBRE_FICHERO, nombre);
		map.put(EXTENSION_FICHERO, extension);
		return map;
	}
	
	@Override
	public IFicherosDAO getDAO() {
		return this.ficherosDAO;
	}

}
