package es.fileStore.utils;

import java.io.IOException;
import java.util.Date;

import javax.annotation.Resource;
import javax.mail.internet.InternetAddress;
import javax.mail.internet.MimeMessage;

import org.springframework.core.io.ByteArrayResource;
import org.springframework.mail.javamail.JavaMailSender;
import org.springframework.mail.javamail.MimeMessageHelper;
import org.springframework.ui.freemarker.FreeMarkerTemplateUtils;

import freemarker.template.Configuration;
import freemarker.template.Template;

public class MailService {
	
	@Resource(name="mailSender")
	private JavaMailSender mailsender;
	
	@Resource(name="freeMarkerConfiguration")
	private Configuration freeMarkerConfig;
	
	private String mail;
	private String personal;
	
	
	public MimeMessage getMimeMessage() throws Exception {

		MimeMessage message = null;
		
		try {
			
			message = this.mailsender.createMimeMessage();
			
			message.setSentDate(new Date());
			InternetAddress address = new InternetAddress(mail, personal);
			message.setFrom(address);
			
			
		} catch (Exception e) {
			System.out.print(e.getMessage());
			throw e;
		}
		
		return message;
	}

	
	public void send(MimeMessage message) {
		
		this.mailsender.send(message);
		
	}
	

	public void setMail(String mail) {
		this.mail = mail;
	}


	public void setPersonal(String personal) {
		this.personal = personal;
	}
	
	
	public String processTemplate(String templateName, Object model) throws Exception {
		
		String out = "";
		
		try {
			
			Template template = this.freeMarkerConfig.getTemplate(templateName);
		
			out = FreeMarkerTemplateUtils.processTemplateIntoString(template, model);
		
		} catch (IOException e) {
			System.out.print(e.getMessage());
			throw e;
		}
		
		return out;
	}
	
	
	public void enviarMail(String email, String asunto, Object model, String template, byte[] adjunto, String fileName) throws Exception {
		
		System.out.print("Enviando correo...");
		
		String texto = this.processTemplate(template, model);
		
		MimeMessage message = this.getMimeMessage();

		MimeMessageHelper messageHelper = null;
		
		if ( adjunto != null ) {
			
			messageHelper = new MimeMessageHelper(message, true);

			ByteArrayResource attachment = new ByteArrayResource(adjunto);
			
			messageHelper.addAttachment(fileName + ".pdf", attachment, "application/pdf");
			
		} else {
			messageHelper = new MimeMessageHelper(message);
		}
		
		messageHelper.setText(texto, true);
		messageHelper.setTo(new InternetAddress(email));
		messageHelper.setSubject(asunto);
		
		this.send(message);
		
	}
	
}
