package es.fileStore.view.bean;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;

import javax.annotation.processing.FilerException;


public class OnMemoryUploadedFile implements UploadedFile{
	
	private String fileName;
	private InputStream inputStream;
	
	public OnMemoryUploadedFile(File file) throws FileNotFoundException,IOException{
		//longitud del fichero
    	long fileLength = file.length();
        if(Integer.MAX_VALUE<fileLength){
        	throw new FilerException("El tamao del fichero es demasiado grande para ser almacenado en memoria");
        }
        //array de bytes que almacenar el fichero.
        byte arFile[] = new byte[(int)fileLength];
        
        // Read in the bytes 
        int offset = 0; 
        int numRead = 0; 
        InputStream is = new FileInputStream(file);
        //leemos mientras la posicin del byte a leer sea menor que la longitud del array
        //y el nmero de bytes leidos (numRead) sea mayor de 0.
        while (offset < arFile.length &&
        		(numRead=is.read(arFile, offset, arFile.length-offset)) >= 0) { 
        	offset += numRead; 
        }
        is.close();
        
        /* una vez que tenemos el fichero en un array de bytes, creamos un inputStream
         * para su posterior lectura
         */
        inputStream = new ByteArrayInputStream(arFile);
        //nombre del fichero
        this.fileName = file.getName();
	}
	
	public OnMemoryUploadedFile(byte[]fileBytes,String fileName) {
		this.inputStream = new ByteArrayInputStream(fileBytes);
		this.fileName = fileName;
	}
	
	public String getFileName() {
		return fileName;
	}
	
	public InputStream getInputStream() {
		return inputStream;
	}

}
