package es.fileStore.view.controller.ficheros;

import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.TimeZone;

import javax.annotation.PostConstruct;
import javax.faces.event.ValueChangeEvent;
import javax.faces.model.SelectItem;



import com.icesoft.faces.component.datapaginator.DataPaginator;
import com.icesoft.faces.component.ext.HtmlDataTable;
import com.icesoft.faces.context.ByteArrayResource;
import com.icesoft.faces.context.Resource;

import es.fileStore.bean.FiltroCategorias;
import es.fileStore.bean.FiltroFicheros;
import es.fileStore.entity.Categorias;
import es.fileStore.entity.Ficheros;
import es.fileStore.utils.ConfigurationManager;
import es.fileStore.view.bean.FicheroBean;
import es.fileStore.view.controller.BaseController;

public class FicherosController extends BaseController implements Serializable {
	
	
	private static final long serialVersionUID = 1L;
	
	private Integer idCategoria;
	private String texto;
	
	private boolean mostrarPaginador = false;
	private boolean sinDatos = false;
	private boolean mostrarListado = false;
	private boolean mostrarDescarga = false;
	private List<FicheroBean> ficheros;
	
	private DataPaginator paginador;
	private HtmlDataTable datatable;
	
	
	//marca qu fichero est visualizando el usuario.
	private Ficheros ficheroDetalle = null;
	
	@PostConstruct
	public void init() throws Exception {
		
		this.idCategoria = null;
		this.texto = null;
		this.mostrarPaginador = false;
		this.mostrarListado = false;
		this.ficheros = null;
		this.sinDatos = false;
		this.ficheroDetalle = null;
		this.mostrarDescarga = false;
		
	}
	
	public List<SelectItem> getCategorias() throws Exception {
		
		List<SelectItem> categorias = null;
		
		
		FiltroCategorias filtro = new FiltroCategorias();
				
		List<Categorias> listaCategorias = super.getBusinessDBLocator().getCategoriasLN().getListaCategorias(filtro );
		
		if ( listaCategorias != null && listaCategorias.size() > 0 ) {
			
			categorias = new ArrayList<SelectItem>();
			
			for ( Categorias cat : listaCategorias ) {
				
				categorias.add(new SelectItem(cat.getIdCategoria(), cat.getNombreCategoria()));
			}
		}
		
		return categorias;
	}
	
	
	public String buscar() throws Exception {
		
		FiltroFicheros filtro = new FiltroFicheros();
		
				
		filtro.setIdCategoria(this.idCategoria);
		filtro.setTexto(this.texto);
		
		this.ficheros = super.getBusinessDBLocator().getFicherosLN().consulta(filtro);
		
		this.sinDatos = false;
		
		if ( this.ficheros != null && this.ficheros.size() > 0 ) {
			
			if ( this.ficheros.size() <= 10 )
				this.mostrarPaginador = false;
			else
				this.mostrarPaginador = true;
			
			if ( this.ficheros.size() > 0 )
			{
				this.mostrarListado = true;
				this.datatable = new HtmlDataTable();
				this.paginador = new DataPaginator();
			}
			else
				this.mostrarListado = false; 
		} else {
			
			this.sinDatos = true;
			this.mostrarListado = false;
			this.mostrarPaginador = false;
		}
		
		return "consultaFicheros";
	}
	
	public String volverTabla()
	{
		return "consultaFicheros";
	}

 	public String detalle() throws Exception{
		
		Integer idFichero = Integer.valueOf(super.getRequestParam("idFichero"));
		
		Ficheros fichero = super.getBusinessDBLocator().getFicherosLN().detalle(idFichero);
		
		//primero hacemos una copia del fichero, para que no se modifique en BD
		Ficheros fCopia = (Ficheros)fichero.clone();
		//guardamos el fichero que ha solicitado ver el usuario al detalle, por si decide descargarlo
		
		ficheroDetalle = fichero;
		//eliminamos el id para que hibernate no modifique el estado de este objeto en BD
		fCopia.setIdFichero(null);
		
		
		StringBuilder largeName = new StringBuilder(fCopia.getRutaFichero());
		int underScorePos = largeName.lastIndexOf("_");
		int dotPos = largeName.lastIndexOf(".");
		if(dotPos==-1){//por si el fichero no tiene extensin
			dotPos = largeName.length();
		}
		largeName.replace(underScorePos, dotPos, "");
		fCopia.setRutaFichero(largeName.toString());
		
		super.setRequestParam("ficheroD", fCopia);

		return "detalleFichero";
	}
	
 	
	public Resource getDownloadFile() throws Exception{		
		Ficheros fichero = super.getBusinessDBLocator().getFicherosLN().detalle(ficheroDetalle.getIdFichero());
		String rutaFicheros = ConfigurationManager.getProperty("ruta_ficheros");
		
		
		String rutaCompletaFichero = rutaFicheros+"/"+fichero.getRutaFichero();
		
		File file = new File(rutaCompletaFichero);
		
		if (file.exists()){
		
			FileInputStream fi = new FileInputStream(file);
			//array de bytes que almacenar el fichero
	        byte arFile[] = new byte[(int)file.length()];
	        
	        // Read in the bytes 
	        int offset = 0; 
	        int numRead = 0; 
	        InputStream is = new FileInputStream(file);
	        //leemos mientras la posicin del byte a leer sea menor que la longitud del array
	        //y el nmero de bytes leidos (numRead) sea mayor de 0
	        while (offset < arFile.length &&
	        		(numRead=is.read(arFile, offset, arFile.length-offset)) >= 0) { 
	        	offset += numRead; 
	        }
	        is.close();
	        fi.close();
			return new ByteArrayResource(arFile);
		}
		else
			return null;
		
	}
	
	
	public void change(ValueChangeEvent e) {
		
		this.setMostrarListado(false);
		this.setMostrarPaginador(false);
	}

	public TimeZone getTimeZone() {
		return TimeZone.getDefault();
	}
	
	
	public Integer getIdCategoria() {
		return idCategoria;
	}


	public void setIdCategoria(Integer idCategoria) {
		this.idCategoria = idCategoria;
	}
	
	public boolean isMostrarPaginador() {
		return mostrarPaginador;
	}


	public void setMostrarPaginador(boolean mostrarPaginador) {
		this.mostrarPaginador = mostrarPaginador;
	}


	public boolean isMostrarListado() {
		return mostrarListado;
	}


	public void setMostrarListado(boolean mostrarListado) {
		this.mostrarListado = mostrarListado;
	}


	public boolean isSinDatos() {
		return sinDatos;
	}


	public void setSinDatos(boolean sinDatos) {
		this.sinDatos = sinDatos;
	}

	public List<FicheroBean> getFicheros() {
		return ficheros;
	}

	public void setFicheros(List<FicheroBean> ficheros) {
		this.ficheros = ficheros;
	}

	public boolean isMostrarDescarga() {
		return mostrarDescarga;
	}

	public void setMostrarDescarga(boolean mostrarDescarga) {
		this.mostrarDescarga = mostrarDescarga;
	}

	public String getTexto() {
		return texto;
	}

	public void setTexto(String texto) {
		this.texto = texto;
	}

	public Ficheros getFicheroDetalle() {
		return ficheroDetalle;
	}

	public void setFicheroDetalle(Ficheros ficheroDetalle) {
		this.ficheroDetalle = ficheroDetalle;
	}

	public DataPaginator getPaginador() {
		return paginador;
	}

	public void setPaginador(DataPaginator paginador) {
		this.paginador = paginador;
	}

	public HtmlDataTable getDatatable() {
		return datatable;
	}

	public void setDatatable(HtmlDataTable datatable) {
		this.datatable = datatable;
	}

}
