package es.fileStore.view.controller.ficheros;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.EventObject;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.annotation.Resource;
import javax.faces.application.FacesMessage;
import javax.faces.context.FacesContext;
import javax.faces.event.ActionEvent;
import javax.faces.model.SelectItem;
import javax.mail.internet.InternetAddress;
import javax.mail.internet.MimeMessage;

import org.apache.commons.lang.StringUtils;
import org.springframework.mail.javamail.MimeMessageHelper;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import com.icesoft.faces.component.ext.HtmlOutputText;
import com.icesoft.faces.component.inputfile.FileInfo;
import com.icesoft.faces.component.inputfile.InputFile;
import com.icesoft.faces.component.inputfile.InputFileProgressEvent;

import es.fileStore.bean.FiltroCategorias;
import es.fileStore.entity.Categorias;
import es.fileStore.entity.Ficheros;
import es.fileStore.utils.ConfigurationManager;
import es.fileStore.view.bean.OnMemoryUploadedFile;
import es.fileStore.view.controller.BaseController;
import es.fileStore.utils.MailService;

public class FicherosEnvioController extends BaseController implements Serializable {

	private static final long serialVersionUID = 1L;
	
	@Resource(name="mailService")
	private MailService mailService;

	private HtmlOutputText binding;
 	
	private String titulo;
	private String descripcion;
	private String usuarioAlta;
	private Integer codCategoria;
	private String email;
	
	// Aadir Categora
	
	private String nombreCategoria;
	
	//almacena, en memoria, el fichero subido
	private OnMemoryUploadedFile uploadedFile;
	
	private Ficheros fichero;
	private Boolean fieldsFilled = false;
	
	private String labelCat;//muestra el nombre de la categora seleccionada
	
	private boolean extensionPermitida;
	
	private int progreso;
	private boolean overSize;
	
	
	public String init() throws Exception {
		
		this.codCategoria = null;
		this.descripcion = null;
		this.titulo = null;
		this.usuarioAlta=null;
		uploadedFile = null;
		fichero = null;
		fieldsFilled = false;
		this.progreso = 0;
		this.overSize = false;
		
		return "envioFichero";
	}
	
	public String nuevaCategoria() throws Exception {
		
		this.nombreCategoria=null;
		
		return "nuevaCategoria";
	}
	
	@Transactional(propagation=Propagation.REQUIRED, rollbackFor=Exception.class)
	public String altaCategoria() throws Exception {
		
		/////////////////////////
		mensajeControl = "error";
		/////////////////////////
			
		boolean correcto = true;
			
		if (nombreCategoria==null || (nombreCategoria!=null && nombreCategoria.trim().equals(""))){
			correcto = false;
			super.addMenssage("mensajes", new FacesMessage("El campo NOMBRE CATEGORIA es obligatorio."));
		}
			
		if (correcto){
				
			FiltroCategorias filtro = new FiltroCategorias();
				
			List<Categorias> listaCategorias = super.getBusinessDBLocator().getCategoriasLN().getListaCategorias(filtro );
				
			if ( listaCategorias != null && listaCategorias.size() > 0 ) {
					
				for ( Categorias c : listaCategorias ) {
					
					if (nombreCategoria.trim().equals(c.getNombreCategoria()))
					{
						super.addMenssage("mensajes", new FacesMessage("Ya existe una categora con ese Nombre."));
						return null;
					}
				}
			}
			
			
			
			Categorias categoria = null;
			categoria = new Categorias();
			categoria.setNombreCategoria(nombreCategoria.trim());
		
			super.getBusinessDBLocator().getCategoriasLN().altaCategoria(categoria);
				
			super.addMenssage("mensajes", new FacesMessage("La categora se ha creado correctamente."));
				
		} else return null;
			
		return "envioFichero";
	}
	
	
	public String volverNuevaCategoria() throws Exception {
		return "envioFichero";
	}


	public String alta() throws Exception {
		
		/////////////////////////
		mensajeControl = "error";
		/////////////////////////
		
		boolean error = false;
			
		//Validaciones
		
		
		if ( StringUtils.isEmpty(this.titulo) ) {
			super.addMenssage("mensajes", new FacesMessage("El campo Ttulo es obligatorio."));
			error = true;
		}
		
		if ( StringUtils.isEmpty(this.descripcion) ) {
			super.addMenssage("mensajes", new FacesMessage("El campo Descripcion es obligatorio."));
			error = true;
		}
		
		if ( StringUtils.isEmpty(this.usuarioAlta) ) {
			super.addMenssage("mensajes", new FacesMessage("El campo Usuario es obligatorio."));
			error = true;
		}
		
		if ( this.codCategoria == 0 ) {
			super.addMenssage("mensajes", new FacesMessage("El campo Categora es obligatorio."));
			error = true;
		}

		
		if ( error )
			return "envioFichero";
		
		fichero = new Ficheros();
		
		fichero.setTitulo(this.titulo);
		fichero.setDescripcion(this.descripcion);
		fichero.setUsuarioAlta(this.usuarioAlta);
		Categorias categoria = new Categorias();
		categoria.setIdCategoria(this.getCodCategoria());
		fichero.setCategoria(categoria);
		this.labelCat = super.getBusinessDBLocator().getCategoriasLN().getCategoria(this.getCodCategoria()).getNombreCategoria();
		
		//la ruta del fichero se completa en la capa de servicio
		
		fieldsFilled = true;//subimos el flag que indica que los campos se rellenaron
		
		return "envioFichero";
		
	}
	
	public List<SelectItem> getCategorias() throws Exception {
		
		List<SelectItem> categorias = null;
		
		FiltroCategorias filtro = new FiltroCategorias();
		
		List<Categorias> listaCategorias = super.getBusinessDBLocator().getCategoriasLN().getListaCategorias(filtro );
		
		if ( listaCategorias != null && listaCategorias.size() > 0 ) {
		
			categorias = new ArrayList<SelectItem>();
			
			SelectItem item = null;
			
			for ( Categorias c : listaCategorias ) {
				
				item = new SelectItem(c.getIdCategoria(), c.getNombreCategoria());
				categorias.add(item);
			}
		}
		
		return categorias;
	}
	
	public String volver() throws Exception
	{
		return "consultaFicheros";
	}
	
	/*
	 * Para la barrra de progreso Subiendo archivo
	 */
	public void progressUploadFile(EventObject event)
	{
	
		InputFile ifile = (InputFile) event.getSource();
		progreso = ifile.getFileInfo().getPercent();
	
	}
	
	
	/*
	 * realiza la subida del fichero y lo almacena en BD
	 * Se invoca con el control <ice:inputFile..>
	 */
	public void uploadFile(ActionEvent event) throws Exception{
		
		/////////////////////////
		mensajeControl = "error";
		/////////////////////////
		
        InputFile inputFile = (InputFile) event.getSource();
        FileInfo fileInfo = inputFile.getFileInfo();
        overSize=false;
        if (fileInfo.getStatus() == FileInfo.SAVED) {
        	
        	//comprobamos que la extensin esta permitida
        	String fileName = fileInfo.getFileName();
        	
        	int indicePunto = fileName.lastIndexOf(".");
        	
        	String extension = fileName.substring(indicePunto + 1, fileName.length());
        	
        	List<String> extensiones = ConfigurationManager.getList("file.extensions");
        	
        	extensionPermitida = false;
        	
        	
        	for ( String ext : extensiones) {
        		
        		if ( ext.toUpperCase().equals(extension.toUpperCase()) ) {
        			extensionPermitida = true;
        			break;
        		}
        	}

        	
        	if ( extensionPermitida ) {
        		//almacenamos el fichero en memoria
        		uploadedFile =new OnMemoryUploadedFile(fileInfo.getFile());
        	} else {
        		String mensaje = "Slo se permiten las siguientes extensiones ";
        		for ( String ext : extensiones) {
            		mensaje += ext + ";";
            	}
        		super.addMenssage("mensajes",new FacesMessage(mensaje));
        		uploadedFile = null;
        		progreso = 0;
        	}
        	
            //por ltimo, borramos el fichero de la carpeta upload
    		
            if(fileInfo.getFile().delete()){
            	// Correcto
            }else{
            	// TODO El fichero no se ha conseguido borrar.
            }
            
        }else if(FileInfo.SIZE_LIMIT_EXCEEDED == fileInfo.getStatus()){
        	super.addMenssage("mensajes",new FacesMessage("El fichero es demasiado grande"));
        	overSize=true;
        	uploadedFile = null;
        }
    } 
	
	public String checkUploadOK() throws Exception{
		if(uploadedFile==null){
			
        	return "errorEnvio";
        	
		}else{
			
			//la subida del fichero fue correcta
			
			FacesContext.getCurrentInstance().renderResponse();
			
			return "confirmaEnvioFichero";
		}	
	}
	
	@Transactional(propagation=Propagation.REQUIRED, rollbackFor=Exception.class)
	public String confirmarEnvio()throws Exception{
		
		/////////////////////////
		mensajeControl = "error";
		/////////////////////////
		
		super.getBusinessDBLocator().getFicherosLN().altaFichero(fichero,uploadedFile);
		
		super.addMenssage("mensajes",new FacesMessage("El fichero se envi correctamente. Puede seguir enviando ficheros si lo desea"));
		/////////////////////////
		mensajeControl = "aviso";
		/////////////////////////
		
		if(this.email != null && !"".equals(this.email))
		{
			Map<String, String> model = new HashMap<String, String>(0);
			model.put("titulo", this.titulo);
			model.put("descripcion", this.descripcion);
			model.put("usuarioAlta", this.usuarioAlta);
			model.put("categoria", this.labelCat);
			model.put("fichero",this.uploadedFile.getFileName());
			
			try
			{
				this.enviarMail(this.email, ConfigurationManager.getProperty("asunto.mail.Enviofichero"), ConfigurationManager.getProperty("plantilla.aviso.fichero"),model);
				super.addMenssage("mensajes",new FacesMessage("Mensaje de aviso de nuevo fichero mandado a la direccin: " + this.email + "."));
			} catch(Exception e)
			{
				super.addMenssage("mensajes",new FacesMessage("No se ha podido enviar el mensaje de aviso a la direccin: " + this.email ));
				e.printStackTrace();
			}
		}
		
		fichero = null;
		uploadedFile = null;
		fieldsFilled = false;
		this.titulo = null;
		fieldsFilled = false;
		this.progreso = 0;
		this.overSize = false;
		
		return "envioFichero";
	}
	
	
	private void enviarMail(String email, String asunto, String nombrePlantilla,Object model) throws Exception {
		
		String texto = this.mailService.processTemplate(nombrePlantilla, model);
			
		MimeMessage message = mailService.getMimeMessage();
			
		MimeMessageHelper messageHelper = new MimeMessageHelper(message);
			
		messageHelper.setText(texto, true);
		messageHelper.setTo(new InternetAddress(email));
		messageHelper.setSubject(asunto);
			
		mailService.send(message);
		
	}
	
	public String modificarFicheroEnvio()throws Exception{
		uploadedFile = null;
		this.progreso = 0;
		return "envioFichero";
	}
	
	public String modificarDatosFicheroEnvio()throws Exception{
		fieldsFilled = false;
		return "envioFichero";
	}
	
	public void progressEvent(InputFileProgressEvent event){
		/////////////////////////
		mensajeControl = "error";
		/////////////////////////
		InputFile file = (InputFile)event.getSource();
		if(file.getFileInfo().isFailed()){
			super.addMenssage("mensajes",new FacesMessage("El fichero es demasiado grande"));
		}else if(file.getFileInfo().isFinished()){
			binding.setRendered(true);
		}
		
	}
	
			
	
	public String getTitulo() {
		return titulo;
	}


	public void setTitulo(String titulo) {
		this.titulo = titulo;
	}


	public String getUsuarioAlta() {
		return usuarioAlta;
	}


	public void setUsuarioAlta(String usuarioAlta) {
		this.usuarioAlta = usuarioAlta;
	}


	public Integer getCodCategoria() {
		return codCategoria;
	}


	public void setCodCategoria(Integer codCategoria) {
		this.codCategoria = codCategoria;
	}

	public String getDescripcion() {
		return descripcion;
	}

	public void setDescripcion(String descripcion) {
		this.descripcion = descripcion;
	}
	
	public void setUploadedFile(OnMemoryUploadedFile uploadedFile) {
		this.uploadedFile = uploadedFile;
	}
	
	public OnMemoryUploadedFile getUploadedFile() {
		return uploadedFile;
	}
	
	public void setFieldsFilled(Boolean fieldsFilled) {
		this.fieldsFilled = fieldsFilled;
	}
	
	public Boolean getFieldsFilled() {
		return fieldsFilled;
	}
	
	public void setFichero(Ficheros fichero) {
		this.fichero = fichero;
	}
	
	public Ficheros getFichero() {
		return fichero;
	}

	public String getLabelCat() {
		return labelCat;
	}


	public void setLabelCat(String labelCat) {
		this.labelCat = labelCat;
	}


	public boolean getExtensionPermitida() {
		return extensionPermitida;
	}


	public void setExtensionPermitida(boolean extensionPermitida) {
		this.extensionPermitida = extensionPermitida;
	}

	public int getProgreso() {
		return progreso;
	}


	public void setProgreso(int progreso) {
		this.progreso = progreso;
	}


	public boolean isOverSize() {
		return overSize;
	}


	public void setOverSize(boolean overSize) {
		this.overSize = overSize;
	}


	public String getEmail() {
		return email;
	}


	public void setEmail(String email) {
		this.email = email;
	}

	public String getNombreCategoria() {
		return nombreCategoria;
	}

	public void setNombreCategoria(String nombreCategoria) {
		this.nombreCategoria = nombreCategoria;
	}
	
	
	
	
}
